// Copyright 2017 The Johns Hopkins University Applied Physics Laboratory.
// Licensed under the MIT License. See LICENSE.txt in the project root for full license information.

#pragma once

#include <eopcc/cloud.hpp>
#include <eopcc/grid_structs.hpp>

namespace eopcc {

/**
 *  This is the primary completeness metric and the one used for the metrics evaluation. It operates on a post
 *  registration input GridModel that has been genrated from taking the Z-max of a registered PointCloud for each cell
 *  of truth data.
 * Completeness is calculated as follows:
 * For each ground truth point it checks if in a 3x3 neighborhood exists a solution sample such that z value is smaller than
 * a threshold, if it does then completeness count is increased by 1. Note that the closest value in the neighborhood (in z-axis)
 * with respect to the ground truth is selected towards completeness.
 */
double Completeness(const GridModel<double>& input, const GridModel<double>& z_truth, double threshold);

/**
 * Returns the Mean Z error between each cell in the input and truth. This is not directly
 * used as a metric, but is used to find the Z shift during registration.
 */
double ZMeanError(const GridModel<double>& input, const GridModel<double>& z_truth);

/**
 * Returns the RMS Z error between each cell in the input and truth. Also takes an optional
 * zshift argument which is applied to the input before calculating the RMS error.
 */
double ZRMSError(const GridModel<double>& input, const GridModel<double>& z_truth, double zshift=0.0);

/**
 * Returns the Median Z error between each cell in the input and truth. Also takes an optional
 * zshift argument which is applied to the input before calculating the Median error.
 * median_buffer is provided as a scratch pad to perform a 50% n_select without requiring new allocations.
 */
double ZMedianError(const GridModel<double>& input, const GridModel<double>& z_truth, std::vector<double>& median_buffer,
                    double zshift=0.0, bool use_abs=true);

/**
 * Calculates a Cumulative Histogram of Z errors between each cell of input and truth.
 * This is used for manual debugging and deep introspection and is not used as an automated metric.
 */
std::vector<double> ZCumulativeHistogram(const GridModel<double>& input, const GridModel<double>& z_truth,
                                         const std::vector<double>& errorTHs, double zshift=0.0);

/**
 * Calculates each percentile for the given list of errorPercentiles.
 * This is used for manual debugging and deep introspection and is not used as an automated metric.
 */
std::vector<double> ZErrorPercentile(const GridModel<double>& input, const GridModel<double>& z_truth,
                                     const std::vector<double>& errorPercentile, double zshift=0.0);

/**
 * Returns a grid of cells with the squared error between input and truth in each cell.
 * This is used for manual debugging and deep introspection and is not used as an automated metric.
 */
GridModel<double> ZRMSErrorGrid(const GridModel<double>& input, const GridModel<double>& z_truth, double zshift=0.0);

} // namespace eopcc
